import _extends from "@babel/runtime/helpers/esm/extends";
import { isDocumentVisible, isOnline, noop, parseFilterArgs, parseQueryArgs, uniq } from './utils';
import { QueriesObserver } from './queriesObserver';
import { QueryObserver } from './queryObserver';
import { initFocusHandler } from './focusHandler';
import { initOnlineHandler } from './onlineHandler';
import { notifyManager } from './notifyManager'; // TYPES

// CLASS
export var QueryClient = /*#__PURE__*/function () {
  function QueryClient(config) {
    this.cache = config.cache;
    this.defaultOptions = config.defaultOptions || {};
  }

  var _proto = QueryClient.prototype;

  _proto.getDefaultOptions = function getDefaultOptions() {
    return this.defaultOptions;
  };

  _proto.setDefaultOptions = function setDefaultOptions(options) {
    this.defaultOptions = options;
  };

  _proto.mount = function mount() {
    mountedClients.push(this);
    initFocusHandler(onFocus);
    initOnlineHandler(onOnline);
  };

  _proto.unmount = function unmount() {
    var index = mountedClients.indexOf(this);

    if (index > -1) {
      mountedClients.splice(index, 1);
    }
  };

  _proto.isFetching = function isFetching(arg1, arg2) {
    var _parseFilterArgs = parseFilterArgs(arg1, arg2),
        filters = _parseFilterArgs[0];

    filters.fetching = true;
    return this.cache.findAll(filters).length;
  };

  _proto.setQueryDefaults = function setQueryDefaults(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
    var defaultedOptions = this.defaultQueryOptions(parsedOptions);
    this.cache.build(defaultedOptions).setDefaultOptions(defaultedOptions);
  };

  _proto.getQueryData = function getQueryData(queryKey, filters) {
    var _this$cache$find;

    return (_this$cache$find = this.cache.find(queryKey, filters)) == null ? void 0 : _this$cache$find.state.data;
  };

  _proto.setQueryData = function setQueryData(queryKey, updater, options) {
    var parsedOptions = parseQueryArgs(queryKey);
    var defaultedOptions = this.defaultQueryOptions(parsedOptions);
    return this.cache.build(defaultedOptions).setData(updater, options);
  };

  _proto.getQueryState = function getQueryState(queryKey, filters) {
    var _this$cache$find2;

    return (_this$cache$find2 = this.cache.find(queryKey, filters)) == null ? void 0 : _this$cache$find2.state;
  };

  _proto.removeQueries = function removeQueries(arg1, arg2) {
    var _this = this;

    notifyManager.batch(function () {
      _this.cache.findAll(arg1, arg2).forEach(function (query) {
        _this.cache.remove(query);
      });
    });
  };

  _proto.cancelQueries = function cancelQueries(arg1, arg2, arg3) {
    var _this2 = this;

    var _parseFilterArgs2 = parseFilterArgs(arg1, arg2, arg3),
        filters = _parseFilterArgs2[0],
        options = _parseFilterArgs2[1];

    var cancelOptions = options || {};

    if (typeof cancelOptions.revert === 'undefined') {
      cancelOptions.revert = true;
    }

    var promises = notifyManager.batch(function () {
      return _this2.cache.findAll(filters).map(function (query) {
        return query.cancel(cancelOptions);
      });
    });
    return Promise.all(promises).then(noop).catch(noop);
  };

  _proto.invalidateQueries = function invalidateQueries(arg1, arg2, arg3) {
    var _filters$refetchActiv,
        _filters$refetchInact,
        _this3 = this;

    var _parseFilterArgs3 = parseFilterArgs(arg1, arg2, arg3),
        filters = _parseFilterArgs3[0],
        options = _parseFilterArgs3[1];

    var refetchFilters = _extends({}, filters, {
      active: (_filters$refetchActiv = filters.refetchActive) != null ? _filters$refetchActiv : true,
      inactive: (_filters$refetchInact = filters.refetchInactive) != null ? _filters$refetchInact : false
    });

    return notifyManager.batch(function () {
      _this3.cache.findAll(filters).forEach(function (query) {
        query.invalidate();
      });

      return _this3.refetchQueries(refetchFilters, options);
    });
  };

  _proto.refetchQueries = function refetchQueries(arg1, arg2, arg3) {
    var _this4 = this;

    var _parseFilterArgs4 = parseFilterArgs(arg1, arg2, arg3),
        filters = _parseFilterArgs4[0],
        options = _parseFilterArgs4[1];

    var promises = notifyManager.batch(function () {
      return _this4.cache.findAll(filters).map(function (query) {
        return query.fetch();
      });
    });
    var promise = Promise.all(promises).then(noop);

    if (!(options == null ? void 0 : options.throwOnError)) {
      promise = promise.catch(noop);
    }

    return promise;
  };

  _proto.watchQuery = function watchQuery(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
    return new QueryObserver({
      client: this,
      options: parsedOptions
    });
  };

  _proto.watchQueries = function watchQueries(queries) {
    return new QueriesObserver({
      client: this,
      queries: queries
    });
  };

  _proto.fetchQueryData = function fetchQueryData(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3); // https://github.com/tannerlinsley/react-query/issues/652

    if (typeof parsedOptions.retry === 'undefined') {
      parsedOptions.retry = false;
    }

    var defaultedOptions = this.defaultQueryOptions(parsedOptions);
    var query = this.cache.find(defaultedOptions.queryKey);

    if (!query) {
      query = this.cache.build(defaultedOptions);
    } else if (!query.isStaleByTime(defaultedOptions.staleTime)) {
      return Promise.resolve(query.state.data);
    }

    return query.fetch(defaultedOptions);
  };

  _proto.prefetchQuery = function prefetchQuery(arg1, arg2, arg3) {
    return this.fetchQueryData(arg1, arg2, arg3).then(noop).catch(noop);
  };

  _proto.getCache = function getCache() {
    return this.cache;
  };

  _proto.defaultQueryOptions = function defaultQueryOptions(options) {
    return _extends({}, this.defaultOptions.queries, options);
  };

  _proto.defaultQueryObserverOptions = function defaultQueryObserverOptions(options) {
    return _extends({}, this.defaultOptions.queries, options);
  };

  _proto.defaultMutationOptions = function defaultMutationOptions(options) {
    return _extends({}, this.defaultOptions.mutations, options);
  };

  return QueryClient;
}();
var mountedClients = [];

function onFocus() {
  onExternalEvent('focus');
}

function onOnline() {
  onExternalEvent('online');
}

function onExternalEvent(type) {
  if (isDocumentVisible() && isOnline()) {
    notifyManager.batch(function () {
      uniq(mountedClients.map(function (x) {
        return x.getCache();
      })).forEach(function (cache) {
        cache.getAll().forEach(function (query) {
          if (type === 'focus') {
            query.onFocus();
          } else {
            query.onOnline();
          }
        });
      });
    });
  }
}