import _extends from "@babel/runtime/helpers/esm/extends";
import { getStatusProps, isDocumentVisible, isServer, isValidTimeout, noop, replaceEqualDeep, shallowEqualObjects, timeUntilStale } from './utils';
import { notifyManager } from './notifyManager';
export var QueryObserver = /*#__PURE__*/function () {
  function QueryObserver(config) {
    this.client = config.client;
    this.options = config.client.defaultQueryObserverOptions(config.options);
    this.listeners = [];
    this.initialDataUpdateCount = 0; // Bind exposed methods

    this.remove = this.remove.bind(this);
    this.refetch = this.refetch.bind(this);
    this.fetchNextPage = this.fetchNextPage.bind(this);
    this.fetchPreviousPage = this.fetchPreviousPage.bind(this); // Subscribe to the query

    this.updateQuery();
  }

  var _proto = QueryObserver.prototype;

  _proto.subscribe = function subscribe(listener) {
    var _this = this;

    var callback = listener || function () {
      return undefined;
    };

    this.listeners.push(callback);

    if (this.listeners.length === 1) {
      this.onMount();
    }

    return function () {
      _this.unsubscribe(callback);
    };
  };

  _proto.unsubscribe = function unsubscribe(listener) {
    this.listeners = this.listeners.filter(function (x) {
      return x !== listener;
    });

    if (!this.listeners.length) {
      this.clear();
    }
  };

  _proto.onMount = function onMount() {
    this.currentQuery.subscribeObserver(this);

    if (this.willFetchOnMount()) {
      this.executeFetch();
    }

    this.updateTimers();
  };

  _proto.willFetchOnMount = function willFetchOnMount() {
    return this.options.enabled !== false && (!this.currentQuery.state.updatedAt || this.options.refetchOnMount === 'always' || this.options.refetchOnMount !== false && this.isStale());
  };

  _proto.willFetchOptionally = function willFetchOptionally() {
    return this.options.enabled !== false && this.isStale();
  };

  _proto.isStale = function isStale() {
    return this.currentQuery.isStaleByTime(this.options.staleTime);
  };

  _proto.clear = function clear() {
    this.listeners = [];
    this.clearTimers();
    this.currentQuery.unsubscribeObserver(this);
  };

  _proto.setOptions = function setOptions(options) {
    var prevOptions = this.options;
    var prevQuery = this.currentQuery;
    this.options = this.client.defaultQueryObserverOptions(options); // Keep previous query key if the user does not supply one

    if (!this.options.queryKey) {
      this.options.queryKey = prevOptions.queryKey;
    }

    this.updateQuery(); // Take no further actions if there are no subscribers

    if (!this.listeners.length) {
      return;
    } // If we subscribed to a new query, optionally fetch and update refetch


    if (this.currentQuery !== prevQuery) {
      this.optionalFetch();
      this.updateTimers();
      return;
    } // Optionally fetch if the query became enabled


    if (this.options.enabled !== false && prevOptions.enabled === false) {
      this.optionalFetch();
    } // Update stale interval if needed


    if (this.options.enabled !== prevOptions.enabled || this.options.staleTime !== prevOptions.staleTime) {
      this.updateStaleTimeout();
    } // Update refetch interval if needed


    if (this.options.enabled !== prevOptions.enabled || this.options.refetchInterval !== prevOptions.refetchInterval) {
      this.updateRefetchInterval();
    }
  };

  _proto.getCurrentResult = function getCurrentResult() {
    return this.currentResult;
  };

  _proto.getCurrentOrNextResult = function getCurrentOrNextResult(options) {
    if (this.currentQuery.isFetching()) {
      return this.getNextResult(options);
    } else if (this.currentResult.isError && (options == null ? void 0 : options.throwOnError)) {
      return Promise.reject(this.currentResult.error);
    }

    return Promise.resolve(this.currentResult);
  };

  _proto.getNextResult = function getNextResult(options) {
    var _this2 = this;

    return new Promise(function (resolve, reject) {
      var unsubscribe = _this2.subscribe(function (result) {
        if (!result.isFetching) {
          unsubscribe();

          if (result.isError && (options == null ? void 0 : options.throwOnError)) {
            reject(result.error);
          } else {
            resolve(result);
          }
        }
      });
    });
  };

  _proto.getCurrentQuery = function getCurrentQuery() {
    return this.currentQuery;
  };

  _proto.remove = function remove() {
    this.currentQuery.remove();
  };

  _proto.refetch = function refetch(options) {
    return this.fetch(options);
  };

  _proto.fetchNextPage = function fetchNextPage(options) {
    return this.fetch({
      throwOnError: options == null ? void 0 : options.throwOnError,
      fetchMore: {
        direction: 'forward',
        pageParam: options == null ? void 0 : options.pageParam
      }
    });
  };

  _proto.fetchPreviousPage = function fetchPreviousPage(options) {
    return this.fetch({
      throwOnError: options == null ? void 0 : options.throwOnError,
      fetchMore: {
        direction: 'backward',
        pageParam: options == null ? void 0 : options.pageParam
      }
    });
  };

  _proto.fetch = function fetch(fetchOptions) {
    if (!this.canFetch()) {
      return this.getCurrentOrNextResult();
    }

    var promise = this.getNextResult(fetchOptions);
    this.executeFetch(fetchOptions);
    return promise;
  };

  _proto.optionalFetch = function optionalFetch() {
    if (this.willFetchOptionally()) {
      this.executeFetch();
    }
  };

  _proto.executeFetch = function executeFetch(fetchOptions) {
    if (this.canFetch()) {
      this.currentQuery.fetch(this.getQueryOptions(), fetchOptions).catch(noop);
    }
  };

  _proto.canFetch = function canFetch() {
    var _this$currentQuery$de;

    return Boolean(this.options.queryFn || ((_this$currentQuery$de = this.currentQuery.defaultOptions) == null ? void 0 : _this$currentQuery$de.queryFn));
  };

  _proto.updateStaleTimeout = function updateStaleTimeout() {
    var _this3 = this;

    this.clearStaleTimeout();

    if (isServer || this.currentResult.isStale || !isValidTimeout(this.options.staleTime)) {
      return;
    }

    var time = timeUntilStale(this.currentResult.updatedAt, this.options.staleTime); // The timeout is sometimes triggered 1 ms before the stale time expiration.
    // To mitigate this issue we always add 1 ms to the timeout.

    var timeout = time + 1;
    this.staleTimeoutId = setTimeout(function () {
      if (!_this3.currentResult.isStale) {
        _this3.updateResult();

        _this3.notify({
          listeners: true,
          cache: true
        });
      }
    }, timeout);
  };

  _proto.updateRefetchInterval = function updateRefetchInterval() {
    var _this4 = this;

    this.clearRefetchInterval();

    if (isServer || this.options.enabled === false || !isValidTimeout(this.options.refetchInterval)) {
      return;
    }

    this.refetchIntervalId = setInterval(function () {
      if (_this4.options.refetchIntervalInBackground || isDocumentVisible()) {
        _this4.executeFetch();
      }
    }, this.options.refetchInterval);
  };

  _proto.updateTimers = function updateTimers() {
    this.updateStaleTimeout();
    this.updateRefetchInterval();
  };

  _proto.clearTimers = function clearTimers() {
    this.clearStaleTimeout();
    this.clearRefetchInterval();
  };

  _proto.clearStaleTimeout = function clearStaleTimeout() {
    clearInterval(this.staleTimeoutId);
    this.staleTimeoutId = undefined;
  };

  _proto.clearRefetchInterval = function clearRefetchInterval() {
    clearInterval(this.refetchIntervalId);
    this.refetchIntervalId = undefined;
  };

  _proto.getQueryOptions = function getQueryOptions() {
    return this.options;
  };

  _proto.updateResult = function updateResult(willFetch) {
    var _this$previousQueryRe;

    var state = this.currentQuery.state;
    var status = state.status,
        isFetching = state.isFetching,
        updatedAt = state.updatedAt;
    var isPreviousData = false;
    var data; // Already set the status to loading if we are going to fetch

    if (willFetch) {
      isFetching = true;

      if (status === 'idle') {
        status = 'loading';
      }
    } // Keep previous data if needed


    if (this.options.keepPreviousData && !state.dataUpdateCount && ((_this$previousQueryRe = this.previousQueryResult) == null ? void 0 : _this$previousQueryRe.isSuccess)) {
      data = this.previousQueryResult.data;
      updatedAt = this.previousQueryResult.updatedAt;
      status = this.previousQueryResult.status;
      isPreviousData = true;
    } else if (this.options.select && typeof state.data !== 'undefined') {
      var _this$currentResultSt;

      // Use the previous select result if the query data did not change
      if (this.currentResult && state.data === ((_this$currentResultSt = this.currentResultState) == null ? void 0 : _this$currentResultSt.data)) {
        data = this.currentResult.data;
      } else {
        data = this.options.select(state.data);

        if (this.options.structuralSharing !== false) {
          var _this$currentResult;

          data = replaceEqualDeep((_this$currentResult = this.currentResult) == null ? void 0 : _this$currentResult.data, data);
        }
      }
    } else {
      data = state.data;
    }

    var result = _extends({}, getStatusProps(status), {
      data: data,
      error: state.error,
      failureCount: state.failureCount,
      fetchNextPage: this.fetchNextPage,
      fetchPreviousPage: this.fetchPreviousPage,
      hasNextPage: state.hasNextPage,
      hasPreviousPage: state.hasPreviousPage,
      isFetched: state.dataUpdateCount > 0,
      isFetchedAfterMount: state.dataUpdateCount > this.initialDataUpdateCount,
      isFetching: isFetching,
      isFetchingNextPage: state.isFetchingNextPage,
      isFetchingPreviousPage: state.isFetchingPreviousPage,
      isPreviousData: isPreviousData,
      isStale: this.isStale(),
      refetch: this.refetch,
      remove: this.remove,
      updatedAt: updatedAt
    }); // Keep reference to the current state on which the current result is based on


    this.currentResultState = state; // Only update if something has changed

    if (!this.currentResult || !shallowEqualObjects(this.currentResult, result)) {
      this.currentResult = result;
    }
  };

  _proto.updateQuery = function updateQuery() {
    var prevQuery = this.currentQuery;
    var queryOptions = this.getQueryOptions();
    var query = this.client.getCache().build(queryOptions);

    if (query === prevQuery) {
      return;
    }

    this.previousQueryResult = this.currentResult;
    this.currentQuery = query;
    this.initialDataUpdateCount = query.state.dataUpdateCount;
    var willFetch = prevQuery ? this.willFetchOptionally() : this.willFetchOnMount();
    this.updateResult(willFetch);

    if (!this.listeners.length) {
      return;
    }

    prevQuery == null ? void 0 : prevQuery.unsubscribeObserver(this);
    this.currentQuery.subscribeObserver(this);

    if (this.options.notifyOnStatusChange !== false) {
      this.notify({
        listeners: true
      });
    }
  };

  _proto.onQueryUpdate = function onQueryUpdate(action) {
    var options = this.options;
    var type = action.type; // Store current result and get new result

    var prevResult = this.currentResult;
    this.updateResult();
    var currentResult = this.currentResult; // Update timers if needed

    if (type === 'success' || type === 'error' || type === 'invalidate') {
      this.updateTimers();
    } // Do not notify if the nothing has changed


    if (prevResult === currentResult) {
      return;
    } // Determine which callbacks to trigger


    var notifyOptions = {};

    if (type === 'success') {
      notifyOptions.onSuccess = true;
    } else if (type === 'error') {
      notifyOptions.onError = true;
    }

    if ( // Always notify if notifyOnStatusChange is set
    options.notifyOnStatusChange !== false || // Otherwise only notify on data or error change
    currentResult.data !== prevResult.data || currentResult.error !== prevResult.error) {
      notifyOptions.listeners = true;
    }

    this.notify(notifyOptions);
  };

  _proto.notify = function notify(notifyOptions) {
    var _this5 = this;

    var options = this.options,
        currentResult = this.currentResult,
        currentQuery = this.currentQuery,
        listeners = this.listeners;
    var onSuccess = options.onSuccess,
        onSettled = options.onSettled,
        onError = options.onError;
    notifyManager.batch(function () {
      // First trigger the configuration callbacks
      if (notifyOptions.onSuccess) {
        if (onSuccess) {
          notifyManager.schedule(function () {
            onSuccess(currentResult.data);
          });
        }

        if (onSettled) {
          notifyManager.schedule(function () {
            onSettled(currentResult.data, null);
          });
        }
      } else if (notifyOptions.onError) {
        if (onError) {
          notifyManager.schedule(function () {
            onError(currentResult.error);
          });
        }

        if (onSettled) {
          notifyManager.schedule(function () {
            onSettled(undefined, currentResult.error);
          });
        }
      } // Then trigger the listeners


      if (notifyOptions.listeners) {
        listeners.forEach(function (listener) {
          notifyManager.schedule(function () {
            listener(currentResult);
          });
        });
      } // Then the cache listeners


      if (notifyOptions.cache) {
        _this5.client.getCache().notify(currentQuery);
      }
    });
  };

  return QueryObserver;
}();