import _extends from "@babel/runtime/helpers/esm/extends";
import React from 'react';
import { useIsMounted } from './utils';
import { getStatusProps, noop } from '../core/utils';
import { getLogger } from '../core/logger';
import { notifyManager } from '../core/notifyManager';
import { useQueryClient } from './QueryClientProvider';

// HOOK
function getDefaultState() {
  return _extends({}, getStatusProps('idle'), {
    data: undefined,
    error: null
  });
}

function reducer(state, action) {
  switch (action.type) {
    case 'reset':
      return getDefaultState();

    case 'loading':
      return _extends({}, getStatusProps('loading'), {
        data: undefined,
        error: null
      });

    case 'success':
      return _extends({}, getStatusProps('success'), {
        data: action.data,
        error: null
      });

    case 'error':
      return _extends({}, getStatusProps('error'), {
        data: undefined,
        error: action.error
      });

    default:
      return state;
  }
}

export function useMutation(mutationFn, options) {
  if (options === void 0) {
    options = {};
  }

  var isMounted = useIsMounted();

  var _React$useReducer = React.useReducer(reducer, null, getDefaultState),
      state = _React$useReducer[0],
      dispatch = _React$useReducer[1];

  var safeDispatch = React.useCallback(function (action) {
    notifyManager.schedule(function () {
      if (isMounted()) {
        dispatch(action);
      }
    });
  }, [dispatch, isMounted]);
  var client = useQueryClient();
  var defaultedOptions = client.defaultMutationOptions(options);
  var lastMutationIdRef = React.useRef(0);
  var lastMutationFnRef = React.useRef(mutationFn);
  lastMutationFnRef.current = mutationFn;
  var lastOptionsRef = React.useRef(defaultedOptions);
  lastOptionsRef.current = defaultedOptions;
  var mutateAsync = React.useCallback(function (vars, mutateOpts) {
    if (mutateOpts === void 0) {
      mutateOpts = {};
    }

    var mutationId = ++lastMutationIdRef.current;
    var mutationOpts = lastOptionsRef.current;
    var lastMutationFn = lastMutationFnRef.current;
    var ctx;
    var data;
    safeDispatch({
      type: 'loading'
    });
    return Promise.resolve().then(function () {
      return mutationOpts.onMutate == null ? void 0 : mutationOpts.onMutate(vars);
    }).then(function (context) {
      ctx = context;
    }).then(function () {
      return lastMutationFn(vars);
    }).then(function (result) {
      data = result;
    }).then(function () {
      return mutationOpts.onSuccess == null ? void 0 : mutationOpts.onSuccess(data, vars, ctx);
    }).then(function () {
      return mutationOpts.onSettled == null ? void 0 : mutationOpts.onSettled(data, null, vars, ctx);
    }).then(function () {
      return mutateOpts.onSuccess == null ? void 0 : mutateOpts.onSuccess(data, vars, ctx);
    }).then(function () {
      return mutateOpts.onSettled == null ? void 0 : mutateOpts.onSettled(data, null, vars, ctx);
    }).then(function () {
      if (lastMutationIdRef.current === mutationId) {
        safeDispatch({
          type: 'success',
          data: data
        });
      }

      return data;
    }).catch(function (error) {
      getLogger().error(error);
      return Promise.resolve().then(function () {
        return mutationOpts.onError == null ? void 0 : mutationOpts.onError(error, vars, ctx);
      }).then(function () {
        return mutationOpts.onSettled == null ? void 0 : mutationOpts.onSettled(undefined, error, vars, ctx);
      }).then(function () {
        return mutateOpts.onError == null ? void 0 : mutateOpts.onError(error, vars, ctx);
      }).then(function () {
        return mutateOpts.onSettled == null ? void 0 : mutateOpts.onSettled(undefined, error, vars, ctx);
      }).then(function () {
        if (lastMutationIdRef.current === mutationId) {
          safeDispatch({
            type: 'error',
            error: error
          });
        }

        throw error;
      });
    });
  }, [safeDispatch]);
  var mutate = React.useCallback(function (variables, mutateOptions) {
    mutateAsync(variables, mutateOptions).catch(noop);
  }, [mutateAsync]);
  var reset = React.useCallback(function () {
    safeDispatch({
      type: 'reset'
    });
  }, [safeDispatch]);
  React.useEffect(function () {
    var lastOptions = lastOptionsRef.current;

    if (state.error && (lastOptions.useErrorBoundary || lastOptions.suspense)) {
      throw state.error;
    }
  }, [state.error]);
  return _extends({}, state, {
    mutate: mutate,
    mutateAsync: mutateAsync,
    reset: reset
  });
}