"use strict";

exports.__esModule = true;
exports.QueriesObserver = void 0;

var _utils = require("./utils");

var _notifyManager = require("./notifyManager");

var QueriesObserver = /*#__PURE__*/function () {
  function QueriesObserver(config) {
    this.client = config.client;
    this.queries = config.queries || [];
    this.result = [];
    this.observers = [];
    this.listeners = []; // Subscribe to queries

    this.updateObservers();
  }

  var _proto = QueriesObserver.prototype;

  _proto.subscribe = function subscribe(listener) {
    var _this = this;

    var callback = listener || function () {
      return undefined;
    };

    this.listeners.push(callback);

    if (this.listeners.length === 1) {
      this.onMount();
    }

    return function () {
      _this.unsubscribe(callback);
    };
  };

  _proto.unsubscribe = function unsubscribe(listener) {
    this.listeners = this.listeners.filter(function (x) {
      return x !== listener;
    });

    if (!this.listeners.length) {
      this.clear();
    }
  };

  _proto.onMount = function onMount() {
    var _this2 = this;

    this.observers.forEach(function (observer) {
      observer.subscribe(function (result) {
        _this2.onUpdate(observer, result);
      });
    });
  };

  _proto.clear = function clear() {
    this.listeners = [];
    this.observers.forEach(function (observer) {
      observer.clear();
    });
  };

  _proto.setQueries = function setQueries(queries) {
    this.queries = queries;
    this.updateObservers();
  };

  _proto.getCurrentResult = function getCurrentResult() {
    return this.result;
  };

  _proto.updateObservers = function updateObservers() {
    var _this3 = this;

    var hasIndexChange = false;
    var prevObservers = this.observers;
    var newObservers = this.queries.map(function (options, i) {
      var observer = prevObservers[i];

      var defaultedOptions = _this3.client.defaultQueryObserverOptions(options);

      var hashFn = (0, _utils.getQueryKeyHashFn)(defaultedOptions);
      defaultedOptions.queryHash = hashFn(defaultedOptions.queryKey);

      if (!observer || observer.getCurrentQuery().queryHash !== defaultedOptions.queryHash) {
        hasIndexChange = true;
        observer = prevObservers.find(function (x) {
          return x.getCurrentQuery().queryHash === defaultedOptions.queryHash;
        });
      }

      if (observer) {
        observer.setOptions(defaultedOptions);
        return observer;
      }

      return _this3.client.watchQuery(defaultedOptions);
    });

    if (prevObservers.length === newObservers.length && !hasIndexChange) {
      return;
    }

    this.observers = newObservers;
    this.result = newObservers.map(function (observer) {
      return observer.getCurrentResult();
    });

    if (!this.listeners.length) {
      return;
    }

    (0, _utils.difference)(prevObservers, newObservers).forEach(function (observer) {
      observer.clear();
    });
    (0, _utils.difference)(newObservers, prevObservers).forEach(function (observer) {
      observer.subscribe(function (result) {
        _this3.onUpdate(observer, result);
      });
    });
    this.notify();
  };

  _proto.onUpdate = function onUpdate(observer, result) {
    var index = this.observers.indexOf(observer);

    if (index !== -1) {
      this.result = (0, _utils.replaceAt)(this.result, index, result);
      this.notify();
    }
  };

  _proto.notify = function notify() {
    var result = this.result,
        listeners = this.listeners;

    _notifyManager.notifyManager.batch(function () {
      listeners.forEach(function (listener) {
        _notifyManager.notifyManager.schedule(function () {
          listener(result);
        });
      });
    });
  };

  return QueriesObserver;
}();

exports.QueriesObserver = QueriesObserver;