"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.Query = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _utils = require("./utils");

var _notifyManager = require("./notifyManager");

var _logger = require("./logger");

// CLASS
var Query = /*#__PURE__*/function () {
  function Query(config) {
    this.setOptions(config.options);
    this.observers = [];
    this.cache = config.cache;
    this.queryKey = config.queryKey;
    this.queryHash = config.queryHash;
    this.state = getDefaultState(this.options);
    this.scheduleGc();
  }

  var _proto = Query.prototype;

  _proto.setOptions = function setOptions(options) {
    var _this$options$cacheTi;

    this.options = (0, _extends2.default)({}, this.defaultOptions, options); // Default to 5 minutes if not cache time is set

    this.cacheTime = Math.max(this.cacheTime || 0, (_this$options$cacheTi = this.options.cacheTime) != null ? _this$options$cacheTi : 5 * 60 * 1000);
  };

  _proto.setDefaultOptions = function setDefaultOptions(options) {
    this.defaultOptions = options;
  };

  _proto.dispatch = function dispatch(action) {
    var _this = this;

    this.state = queryReducer(this.state, action);

    _notifyManager.notifyManager.batch(function () {
      _this.observers.forEach(function (observer) {
        observer.onQueryUpdate(action);
      });

      _this.cache.notify(_this);
    });
  };

  _proto.scheduleGc = function scheduleGc() {
    var _this2 = this;

    this.clearGcTimeout();

    if (_utils.isServer || this.observers.length > 0 || !(0, _utils.isValidTimeout)(this.cacheTime)) {
      return;
    }

    this.gcTimeout = setTimeout(function () {
      _this2.remove();
    }, this.cacheTime);
  };

  _proto.cancel = function cancel(options) {
    var _this$cancelFetch;

    var promise = this.promise || Promise.resolve();
    (_this$cancelFetch = this.cancelFetch) == null ? void 0 : _this$cancelFetch.call(this, options);
    return promise.then(_utils.noop).catch(_utils.noop);
  };

  _proto.clearTimersObservers = function clearTimersObservers() {
    this.observers.forEach(function (observer) {
      observer.clearTimers();
    });
  };

  _proto.clearGcTimeout = function clearGcTimeout() {
    clearTimeout(this.gcTimeout);
    this.gcTimeout = undefined;
  };

  _proto.setData = function setData(updater, options) {
    var _this$options$isDataE, _this$options;

    var prevData = this.state.data; // Get the new data

    var data = (0, _utils.functionalUpdate)(updater, prevData); // Structurally share data between prev and new data if needed

    if (this.options.structuralSharing !== false) {
      data = (0, _utils.replaceEqualDeep)(prevData, data);
    } // Use prev data if an isDataEqual function is defined and returns `true`


    if ((_this$options$isDataE = (_this$options = this.options).isDataEqual) == null ? void 0 : _this$options$isDataE.call(_this$options, prevData, data)) {
      data = prevData;
    } // Set data and mark it as cached


    this.dispatch({
      data: data,
      hasNextPage: hasNextPage(this.options, data),
      hasPreviousPage: hasPreviousPage(this.options, data),
      pageParams: options == null ? void 0 : options.pageParams,
      type: 'success',
      updatedAt: options == null ? void 0 : options.updatedAt
    });
    return data;
  };

  _proto.remove = function remove() {
    this.cache.remove(this);
  };

  _proto.destroy = function destroy() {
    this.clearGcTimeout();
    this.clearTimersObservers();
    this.cancel();
  };

  _proto.isActive = function isActive() {
    return this.observers.some(function (observer) {
      return observer.options.enabled !== false;
    });
  };

  _proto.isFetching = function isFetching() {
    return this.state.isFetching;
  };

  _proto.isStale = function isStale() {
    return this.state.isInvalidated || !this.state.updatedAt || this.observers.some(function (observer) {
      return observer.getCurrentResult().isStale;
    });
  };

  _proto.isStaleByTime = function isStaleByTime(staleTime) {
    if (staleTime === void 0) {
      staleTime = 0;
    }

    return this.state.isInvalidated || !this.state.updatedAt || !(0, _utils.timeUntilStale)(this.state.updatedAt, staleTime);
  };

  _proto.onFocus = function onFocus() {
    this.onExternalEvent('focus');
  };

  _proto.onOnline = function onOnline() {
    this.onExternalEvent('online');
  };

  _proto.onExternalEvent = function onExternalEvent(type) {
    var _this$continueFetch;

    // Execute the first observer that wants to fetch on this event
    var fetchObserver = this.observers.find(function (observer) {
      var _observer$options = observer.options,
          enabled = _observer$options.enabled,
          refetchOnWindowFocus = _observer$options.refetchOnWindowFocus,
          refetchOnReconnect = _observer$options.refetchOnReconnect;

      var _observer$getCurrentR = observer.getCurrentResult(),
          isStale = _observer$getCurrentR.isStale;

      return enabled !== false && (type === 'focus' && (refetchOnWindowFocus === 'always' || refetchOnWindowFocus !== false && isStale) || type === 'online' && (refetchOnReconnect === 'always' || refetchOnReconnect !== false && isStale));
    });

    if (fetchObserver) {
      fetchObserver.refetch();
    } // Continue any paused fetch


    (_this$continueFetch = this.continueFetch) == null ? void 0 : _this$continueFetch.call(this);
  };

  _proto.subscribeObserver = function subscribeObserver(observer) {
    if (this.observers.indexOf(observer) !== -1) {
      return;
    }

    this.observers.push(observer); // Stop the query from being garbage collected

    this.clearGcTimeout();
    this.cache.notify(this);
  };

  _proto.unsubscribeObserver = function unsubscribeObserver(observer) {
    this.observers = this.observers.filter(function (x) {
      return x !== observer;
    });

    if (!this.observers.length) {
      // If the transport layer does not support cancellation
      // we'll let the query continue so the result can be cached
      if (this.isTransportCancelable) {
        this.cancel();
      }

      this.scheduleGc();
    }

    this.cache.notify(this);
  };

  _proto.invalidate = function invalidate() {
    if (!this.state.isInvalidated) {
      this.dispatch({
        type: 'invalidate'
      });
    }
  };

  _proto.fetch = function fetch(options, fetchOptions) {
    var _this3 = this;

    if (this.state.isFetching) if ((fetchOptions == null ? void 0 : fetchOptions.fetchMore) && this.state.updatedAt) {
      // Silently cancel current fetch if the user wants to fetch more
      this.cancel({
        silent: true
      });
    } else if (this.promise) {
      // Return current promise if we are already fetching
      return this.promise;
    } // Update config if passed, otherwise the config from the last execution is used

    if (options) {
      this.setOptions(options);
    } // Use the options from the first observer with a query function if no function is found.
    // This can happen when the query is hydrated or created with setQueryData.


    if (!this.options.queryFn) {
      var observer = this.observers.find(function (x) {
        return x.options.queryFn;
      });

      if (observer) {
        this.setOptions(observer.options);
      }
    } // Get the query function params


    var params = (0, _utils.ensureArray)(this.queryKey);
    var filter = this.options.queryFnParamsFilter;
    params = filter ? filter(params) : params;
    var promise = this.options.infinite ? this.startInfiniteFetch(this.options, params, fetchOptions) : this.startFetch(this.options, params);
    this.promise = promise.catch(function (error) {
      // Set error state if needed
      if (!((0, _utils.isCancelledError)(error) && error.silent)) {
        _this3.dispatch({
          type: 'error',
          error: error
        });
      } // Log error


      if (!(0, _utils.isCancelledError)(error)) {
        (0, _logger.getLogger)().error(error);
      } // Propagate error


      throw error;
    });
    return this.promise;
  };

  _proto.startFetch = function startFetch(options, params) {
    var _this4 = this;

    // Create function to fetch the data
    var queryFn = options.queryFn || defaultQueryFn;

    var fetchData = function fetchData() {
      return queryFn.apply(void 0, params);
    }; // Set to fetching state if not already in it


    if (!this.state.isFetching) {
      this.dispatch({
        type: 'fetch'
      });
    } // Try to fetch the data


    return this.tryFetchData(options, fetchData).then(function (data) {
      return _this4.setData(data);
    });
  };

  _proto.startInfiniteFetch = function startInfiniteFetch(options, params, fetchOptions) {
    var _this5 = this;

    var queryFn = options.queryFn || defaultQueryFn;
    var fetchMore = fetchOptions == null ? void 0 : fetchOptions.fetchMore;
    var pageParam = fetchMore == null ? void 0 : fetchMore.pageParam;
    var isFetchingNextPage = (fetchMore == null ? void 0 : fetchMore.direction) === 'forward';
    var isFetchingPreviousPage = (fetchMore == null ? void 0 : fetchMore.direction) === 'backward';
    var oldPages = this.state.data || [];
    var oldPageParams = this.state.pageParams || [];
    var newPageParams = oldPageParams; // Create function to fetch a page

    var fetchPage = function fetchPage(pages, manual, param, previous) {
      if (typeof param === 'undefined' && !manual && pages.length) {
        return Promise.resolve(pages);
      }

      return Promise.resolve().then(function () {
        return queryFn.apply(void 0, params.concat([param]));
      }).then(function (page) {
        newPageParams = previous ? [param].concat(newPageParams) : [].concat(newPageParams, [param]);
        return previous ? [page].concat(pages) : [].concat(pages, [page]);
      });
    }; // Create function to fetch the data


    var fetchData = function fetchData() {
      // Reset new page params
      newPageParams = oldPageParams; // Fetch first page?

      if (!oldPages.length) {
        return fetchPage([]);
      } // Fetch next page?


      if (isFetchingNextPage) {
        var _manual = typeof pageParam !== 'undefined';

        var param = _manual ? pageParam : getNextPageParam(options, oldPages);
        return fetchPage(oldPages, _manual, param);
      } // Fetch previous page?


      if (isFetchingPreviousPage) {
        var _manual2 = typeof pageParam !== 'undefined';

        var _param = _manual2 ? pageParam : getPreviousPageParam(options, oldPages);

        return fetchPage(oldPages, _manual2, _param, true);
      } // Refetch pages


      newPageParams = [];
      var manual = typeof options.getNextPageParam === 'undefined'; // Fetch first page

      var promise = fetchPage([], manual, oldPageParams[0]); // Fetch remaining pages

      var _loop = function _loop(i) {
        promise = promise.then(function (pages) {
          var param = manual ? oldPageParams[i] : getNextPageParam(options, pages);
          return fetchPage(pages, manual, param);
        });
      };

      for (var i = 1; i < oldPages.length; i++) {
        _loop(i);
      }

      return promise;
    }; // Set to fetching state if not already in it


    if (!this.state.isFetching || this.state.isFetchingNextPage !== isFetchingNextPage || this.state.isFetchingPreviousPage !== isFetchingPreviousPage) {
      this.dispatch({
        type: 'fetch',
        isFetchingNextPage: isFetchingNextPage,
        isFetchingPreviousPage: isFetchingPreviousPage
      });
    } // Try to get the data


    return this.tryFetchData(options, fetchData).then(function (data) {
      return _this5.setData(data, {
        pageParams: newPageParams
      });
    });
  };

  _proto.tryFetchData = function tryFetchData(options, fn) {
    var _this6 = this;

    return new Promise(function (outerResolve, outerReject) {
      var resolved = false;
      var continueLoop;
      var cancelTransport;

      var done = function done() {
        resolved = true;
        delete _this6.cancelFetch;
        delete _this6.continueFetch;
        delete _this6.isTransportCancelable; // End loop if currently paused

        continueLoop == null ? void 0 : continueLoop();
      };

      var resolve = function resolve(value) {
        done();
        outerResolve(value);
      };

      var reject = function reject(value) {
        done();
        outerReject(value);
      }; // Create callback to cancel this fetch


      _this6.cancelFetch = function (cancelOptions) {
        reject(new _utils.CancelledError(cancelOptions));
        cancelTransport == null ? void 0 : cancelTransport();
      }; // Create callback to continue this fetch


      _this6.continueFetch = function () {
        continueLoop == null ? void 0 : continueLoop();
      }; // Create loop function


      var run = function run() {
        // Do nothing if already resolved
        if (resolved) {
          return;
        }

        var promiseOrValue; // Execute query

        try {
          promiseOrValue = fn();
        } catch (error) {
          promiseOrValue = Promise.reject(error);
        } // Check if the transport layer support cancellation


        if ((0, _utils.isCancelable)(promiseOrValue)) {
          cancelTransport = function cancelTransport() {
            try {
              promiseOrValue.cancel();
            } catch (_unused) {}
          };

          _this6.isTransportCancelable = true;
        }

        Promise.resolve(promiseOrValue).then(function (data) {
          // Resolve with data
          resolve(data);
        }).catch(function (error) {
          var _options$retry, _options$retryDelay;

          // Stop if the fetch is already resolved
          if (resolved) {
            return;
          } // Do we need to retry the request?


          var failureCount = _this6.state.failureCount;
          var retry = (_options$retry = options.retry) != null ? _options$retry : 3;
          var retryDelay = (_options$retryDelay = options.retryDelay) != null ? _options$retryDelay : _utils.defaultRetryDelay;
          var shouldRetry = retry === true || typeof retry === 'number' && failureCount < retry || typeof retry === 'function' && retry(failureCount, error);

          if (!shouldRetry) {
            // We are done if the query does not need to be retried
            reject(error);
            return;
          } // Increase the failureCount


          _this6.dispatch({
            type: 'failed'
          }); // Delay


          (0, _utils.sleep)((0, _utils.functionalUpdate)(retryDelay, failureCount) || 0) // Pause if needed
          .then(function () {
            // Pause retry if the document is not visible or when the device is offline
            if (!(0, _utils.isDocumentVisible)() || !(0, _utils.isOnline)()) {
              return new Promise(function (continueResolve) {
                continueLoop = continueResolve;
              });
            }
          }) // Try again
          .then(run);
        });
      }; // Start loop


      run();
    });
  };

  return Query;
}();

exports.Query = Query;

function defaultQueryFn() {
  return Promise.reject();
}

function getNextPageParam(options, pages) {
  return options.getNextPageParam == null ? void 0 : options.getNextPageParam(pages[pages.length - 1], pages);
}

function getPreviousPageParam(options, pages) {
  return options.getPreviousPageParam == null ? void 0 : options.getPreviousPageParam(pages[0], pages);
}
/**
 * Checks if there is a next page.
 * Returns `undefined` if it cannot be determined.
 */


function hasNextPage(options, pages) {
  return options.getNextPageParam && Array.isArray(pages) ? typeof getNextPageParam(options, pages) !== 'undefined' : undefined;
}
/**
 * Checks if there is a previous page.
 * Returns `undefined` if it cannot be determined.
 */


function hasPreviousPage(options, pages) {
  return options.getPreviousPageParam && Array.isArray(pages) ? typeof getPreviousPageParam(options, pages) !== 'undefined' : undefined;
}

function getDefaultState(options) {
  var data = typeof options.initialData === 'function' ? options.initialData() : options.initialData;
  var hasData = typeof data !== 'undefined';
  return {
    data: data,
    dataUpdateCount: 0,
    error: null,
    errorUpdateCount: 0,
    failureCount: 0,
    hasNextPage: hasNextPage(options, data),
    hasPreviousPage: hasPreviousPage(options, data),
    isFetching: false,
    isFetchingNextPage: false,
    isFetchingPreviousPage: false,
    isInvalidated: false,
    pageParams: undefined,
    status: hasData ? 'success' : 'idle',
    updatedAt: hasData ? Date.now() : 0
  };
}

function queryReducer(state, action) {
  var _action$updatedAt;

  switch (action.type) {
    case 'failed':
      return (0, _extends2.default)({}, state, {
        failureCount: state.failureCount + 1
      });

    case 'fetch':
      return (0, _extends2.default)({}, state, {
        failureCount: 0,
        isFetching: true,
        isFetchingNextPage: action.isFetchingNextPage || false,
        isFetchingPreviousPage: action.isFetchingPreviousPage || false,
        status: state.updatedAt ? 'success' : 'loading'
      });

    case 'success':
      return (0, _extends2.default)({}, state, {
        data: action.data,
        dataUpdateCount: state.dataUpdateCount + 1,
        error: null,
        failureCount: 0,
        hasNextPage: action.hasNextPage,
        hasPreviousPage: action.hasPreviousPage,
        pageParams: action.pageParams,
        isFetching: false,
        isFetchingNextPage: false,
        isFetchingPreviousPage: false,
        isInvalidated: false,
        status: 'success',
        updatedAt: (_action$updatedAt = action.updatedAt) != null ? _action$updatedAt : Date.now()
      });

    case 'error':
      var error = action.error;

      if ((0, _utils.isCancelledError)(error) && error.revert) {
        return (0, _extends2.default)({}, state, {
          failureCount: 0,
          isFetching: false,
          isFetchingNextPage: false,
          isFetchingPreviousPage: false,
          status: state.error ? 'error' : state.updatedAt ? 'success' : 'idle'
        });
      }

      return (0, _extends2.default)({}, state, {
        error: error,
        errorUpdateCount: state.errorUpdateCount + 1,
        failureCount: state.failureCount + 1,
        isFetching: false,
        isFetchingNextPage: false,
        isFetchingPreviousPage: false,
        status: 'error'
      });

    case 'invalidate':
      return (0, _extends2.default)({}, state, {
        isInvalidated: true
      });

    default:
      return state;
  }
}